@@ -20,6 +20,8 @@
 #ifndef __XCDRDAO_H__
 #define __XCDRDAO_H__
 
+#include <gtkmm.h>
+
 //
 // GLOBAL objects.
 //
@@ -35,5 +37,21 @@ extern class ConfigManager*      configManager;
 void blockProcessMonitorSignals();
 void unblockProcessMonitorSignals();
 
-#endif
+class GCDMasterApplication: public Gtk::Application
+{
+public:
+GCDMasterApplication() :
+    Gtk::Application("Gonme.CDMaster", Gio::APPLICATION_HANDLES_OPEN) {}
 
+protected:
+    void on_activate() override;
+    void on_open(const Gio::Application::type_vec_files& files,
+                 const Glib::ustring& hint) override;
+
+private:
+    void on_hide_window(Gtk::Window* window);
+};
+
+extern Glib::RefPtr<GCDMasterApplication> app;
+
+#endif
