@@ -35,16 +35,20 @@
 #include <sys/stat.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <sstream>
 
 #include "Toc.h"
 #include "CdTextItem.h"
 #include "util.h"
+#include "log.h"
 
 #include "Cddb.h"
 
+using std::string;
+using std::vector;
 
 #define CDDB_MAX_LINE_LEN 1024
-#define CDDB_DEFAULT_PORT_CDDBP 888
+#define CDDB_DEFAULT_PORT_CDDBP 8880
 #define CDDB_DEFAULT_PORT_HTTP 80
 
 
@@ -57,19 +61,18 @@ static int parseQueryResult(char *line, char *category, char *diskId,
 
 static RETSIGTYPE alarmHandler(int sig)
 {
-  message(0, "ALARM");
+  log_message(0, "ALARM");
 #if RETSIGTYPE != void
   return 0;
 #endif
 }
 
-Cddb::Cddb(const Toc *t)
+Cddb::Cddb(Toc *t)
 {
   toc_ = t;
 
   serverList_ = NULL;
   selectedServer_ = NULL;
-  localCddbDirectory_ = NULL;
 
   fd_ = -1;
   connected_ = 0;
@@ -91,9 +94,6 @@ Cddb::~Cddb()
   clearQueryResults();
   clearCddbEntry();
 
-  delete[] localCddbDirectory_;
-  localCddbDirectory_ = NULL;
-
   delete[] httpCmd_;
   httpCmd_ = NULL;
 
@@ -124,18 +124,17 @@ void Cddb::timeout(int t)
     timeout_ = t;
 }
 
-void Cddb::localCddbDirectory(const char *dir)
+void Cddb::localCddbDirectory(const std::string& dir)
 {
   const char *homeDir;
 
-  delete[] localCddbDirectory_;
-
   // replace ~/ by the path to the home directory as indicated by $HOME
   if (dir[0] == '~' && dir[1] == '/' && (homeDir = getenv("HOME")) != NULL) {
-    localCddbDirectory_ = strdupvCC(homeDir, dir + 1, NULL);
+    localCddbDirectory_ = homeDir;
+    localCddbDirectory_ += dir.substr(1);
   }
   else {
-    localCddbDirectory_ = strdupCC(dir);
+    localCddbDirectory_ = dir;
   }
 }
 
@@ -354,7 +353,7 @@ int Cddb::openConnection()
 #endif
   
   if (sigaction(SIGALRM, &newAlarmHandler, &oldAlarmHandler) != 0) {
-    message(-2, "CDDB: Cannot install alarm signal handler: %s",
+    log_message(-2, "CDDB: Cannot install alarm signal handler: %s",
 	    strerror(errno));
     return 1;
   }
@@ -372,18 +371,18 @@ int Cddb::openConnection()
 	server = run->httpProxyServer;
 	port = run->httpProxyPort;
 
-	message(1,
+	log_message(1,
 		"CDDB: Connecting to http://%s:%u%s via proxy %s:%u ...",
 		run->server, run->port, run->httpCgiBin, server, port);
       }
       else {
-	message(1,
+	log_message(1,
 		"CDDB: Connecting to http://%s:%u%s ...", server, port,
 		run->httpCgiBin);
       }
     }
     else {
-      message(1, "CDDB: Connecting to cddbp://%s:%u ...", server, port);
+      log_message(1, "CDDB: Connecting to cddbp://%s:%u ...", server, port);
     }
 
 #ifdef HAVE_INET_ATON
@@ -394,7 +393,7 @@ int Cddb::openConnection()
       if ((hostEnt = gethostbyname(server)) == NULL ||
 	  hostEnt->h_addrtype != AF_INET) {
 	alarm(0);
-	message(-1, "CDDB: Cannot resolve hostname '%s' - skipping.", server);
+	log_message(-1, "CDDB: Cannot resolve hostname '%s' - skipping.", server);
 	continue;
       }
       else {
@@ -407,11 +406,11 @@ int Cddb::openConnection()
     }
 #endif
 
-    message(4, "CDDB: Hostname: %s -> IP: %s", server,
+    log_message(4, "CDDB: Hostname: %s -> IP: %s", server,
 	    inet_ntoa(sockAddr.sin_addr));
 
     if ((fd_ = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-      message(-2, "CDDB: Cannot create socket: %s", strerror(errno));
+      log_message(-2, "CDDB: Cannot create socket: %s", strerror(errno));
       goto fail;
     }
 
@@ -422,13 +421,13 @@ int Cddb::openConnection()
 
     if (connect(fd_, (struct sockaddr*)&sockAddr, sizeof(sockAddr)) == 0) {
       alarm(0);
-      message(1, "CDDB: Ok.");
+      log_message(1, "CDDB: Ok.");
       selectedServer_ = run;
       break;
     }
     else {
       alarm(0);
-      message(-1, "CDDB: Failed to connect to '%s:%u: %s", server, port,
+      log_message(-1, "CDDB: Failed to connect to '%s:%u: %s", server, port,
 	      strerror(errno));
       closeConnection();
     }
@@ -440,7 +439,7 @@ int Cddb::openConnection()
   alarm(0);
 
   if (sigaction(SIGALRM, &oldAlarmHandler, NULL) != 0) {
-    message(-1, "CDDB: Cannot restore alarm signal handler: %s",
+    log_message(-1, "CDDB: Cannot restore alarm signal handler: %s",
 	    strerror(errno));
   }
   
@@ -472,7 +471,7 @@ void Cddb::setupHttpData(const char *userName, const char *hostName,
 
   httpCmd_ = strdupvCC("&hello=", userName, "+",  hostName, "+",
 		       clientName, "+", version, 
-		       "&proto=1", NULL);
+		       "&proto=6", NULL);
 
   delete[] httpData_;
 
@@ -510,17 +509,17 @@ int Cddb::connectDb(const char *userName, const char *hostName,
   response = getServerResponse(code);
 
   if (response == NULL) {
-    message(-2, "CDDB: EOF while waiting for server greeting.");
+    log_message(-2, "CDDB: EOF while waiting for server greeting.");
     closeConnection();
     return 2;
   }
 
   if (code[0] != 2) {
-    message(-2, "CDDB: Connection to server denied: %s", response);
+    log_message(-2, "CDDB: Connection to server denied: %s", response);
     return 2;
   }
 
-  message(4, "CDDB: Server greeting: %s", response);
+  log_message(4, "CDDB: Server greeting: %s", response);
 
   connected_ = 1;
 
@@ -531,24 +530,32 @@ int Cddb::connectDb(const char *userName, const char *hostName,
   cmdArgs[4] = clientName;
   cmdArgs[5] = version;
 
-  if (sendCommand(6, cmdArgs) != 0) {
-    message(-2, "CDDB: Failed to send handshake command.");
+  if (sendCommand({"cddb", "hello", userName, hostName, clientName, version}) != 0) {
+    log_message(-2, "CDDB: Failed to send handshake command.");
     return 2;
   }
 
   response = getServerResponse(code);
 
   if (response == NULL) {
-    message(-2, "CDDB: EOF while waiting for server handshake response.");
+    log_message(-2, "CDDB: EOF while waiting for server handshake response.");
     return 2;
   }
 
   if (code[0] != 2) {
-    message(-2, "CDDB: Server handshake failed: %s", response);
+    log_message(-2, "CDDB: Server handshake failed: %s", response);
     return 2;
   }
 
-  message(4, "CDDB: Handshake response: %s", response);
+  log_message(4, "CDDB: Handshake response: %s", response);
+
+  if (sendCommand({"proto", "6"}) != 0) {
+    log_message(-2, "CDDB: Failed to send proto command.");
+    return 2;
+  }
+  response = getServerResponse(code);
+
+  log_message(4, "CDDB: Handshake response: %s", response);
 
   return 0;
 }
@@ -614,7 +621,7 @@ bool Cddb::printDbEntry()
 int Cddb::queryDb(QueryResults **results)
 {
   const char *cddbId;
-  const char **args;
+  vector<string> args;
   const char *resp;
   char qtitle[CDDB_MAX_LINE_LEN];
   char qcategory[CDDB_MAX_LINE_LEN];
@@ -623,12 +630,10 @@ int Cddb::queryDb(QueryResults **results)
   char *buf;
   int code[3];
   int err = 0;
-  int nargs;
-  int arg, i;
+  int i;
   int ntracks;
   const Track *t;
   Msf start, end;
-  long diskLength;
 
   // clear previous results
   clearQueryResults();
@@ -640,51 +645,37 @@ int Cddb::queryDb(QueryResults **results)
 
   ntracks = toc_->nofTracks();
 
-  nargs = ntracks + 5;
-
-  args = new const char*[nargs];
-  arg = 0;
-
-  args[arg++] = "cddb";
-  args[arg++] = "query";
+  args = { "cddb", "query" };
 
   cddbId = calcCddbId();
 
-  args[arg++] = cddbId;
-
-  buf = new char[20];
-  sprintf(buf, "%d", ntracks);
-  args[arg++] = buf;
+  args.push_back(cddbId);
+  args.push_back(std::to_string(ntracks));
 
   TrackIterator itr(toc_);
 
-  for (t = itr.first(start, end); t != NULL; t = itr.next(start, end)) {
-    long trackStart = start.lba() + 150;
-    buf = new char[20];
+  for (t = itr.first(start, end); t != NULL; t = itr.next(start, end))
+    args.push_back(std::to_string(start.lba() + 150));
 
-    sprintf(buf, "%ld", trackStart);
-    args[arg++] = buf;
+  {
+    long diskLength = toc_->length().min() * 60 + toc_->length().sec() + 2;
+    args.push_back(std::to_string(diskLength));
   }
-
-  buf = new char[20];
-  diskLength = toc_->length().min() * 60 + toc_->length().sec() + 2;
-  sprintf(buf, "%ld", diskLength);
-  args[arg++] = buf;
-  
-  if (sendCommand(nargs, args) != 0) {
-    message(-2, "CDDB: Failed to send QUERY command.");
+                   
+  if (sendCommand(args) != 0) {
+    log_message(-2, "CDDB: Failed to send QUERY command.");
     err = 1; goto fail;
   }
 
   if ((resp = getServerResponse(code)) == NULL) {
-    message(-2, "CDDB: EOF while waiting for QUERY response.");
+    log_message(-2, "CDDB: EOF while waiting for QUERY response.");
     err = 1; goto fail;
   }    
 
-  message(4, "CDDB: QUERY response: %s", resp);
+  log_message(4, "CDDB: QUERY response: %s", resp);
 
   if (code[0] != 2) {
-    message(-2, "CDDB: QUERY failed: %s", resp);
+    log_message(-2, "CDDB: QUERY failed: %s", resp);
     err = 1; goto fail;
   }
   else {
@@ -695,7 +686,7 @@ int Cddb::queryDb(QueryResults **results)
 	appendQueryResult(qcategory, qdiskId, qtitle, 1);
       }
       else {
-	message(-2, "CDDB: Received invalid QUERY response: %s", resp);
+	log_message(-2, "CDDB: Received invalid QUERY response: %s", resp);
 	err = 1; goto fail;
       }
     }
@@ -705,19 +696,19 @@ int Cddb::queryDb(QueryResults **results)
 	     strcmp(resp, ".") != 0) {
 	strcpy(respBuf, resp);
 
-	message(4, "CDDB: Query data: %s", resp);
+	log_message(4, "CDDB: Query data: %s", resp);
 
 	if (parseQueryResult(respBuf, qcategory, qdiskId, qtitle)) {
 	  appendQueryResult(qcategory, qdiskId, qtitle, 0);
 	}
 	else {
-	  message(-2, "CDDB: Received invalid QUERY data: %s", resp);
+	  log_message(-2, "CDDB: Received invalid QUERY data: %s", resp);
 	  err = 1; goto fail;
 	}
       }
 
       if (resp == NULL) {
-	message(-2, "CDDB: EOF while reading QUERY data.");
+	log_message(-2, "CDDB: EOF while reading QUERY data.");
 	err = 1; goto fail;
       }	
     }
@@ -732,11 +723,6 @@ int Cddb::queryDb(QueryResults **results)
   if (httpMode_)
     closeConnection();
 
-  for (i = 3; i < arg; i++)
-    delete[] args[i];
-
-  delete[] args;
-  
   *results = queryResults_;
   return err;
   
@@ -750,7 +736,6 @@ int Cddb::queryDb(QueryResults **results)
 int Cddb::readDb(const char *category, const char *diskId, CddbEntry **entry)
 {
   int code[3];
-  const char *args[4];
   const char *resp;
   int localRecordFd = -1;
 
@@ -761,34 +746,29 @@ int Cddb::readDb(const char *category, const char *diskId, CddbEntry **entry)
       return 1;
   }
 
-  args[0] = "cddb";
-  args[1] = "read";
-  args[2] = category;
-  args[3] = diskId;
-
-  if (sendCommand(4, args) != 0) {
-    message(-2, "CDDB: Failed to send READ command.");
+  if (sendCommand({"cddb", "read", category, diskId}) != 0) {
+    log_message(-2, "CDDB: Failed to send READ command.");
     goto fail;
   }
   
   if ((resp = getServerResponse(code)) == NULL) {
-    message(-2, "CDDB: EOF while waiting for READ response.");
+    log_message(-2, "CDDB: EOF while waiting for READ response.");
     goto fail;
   }
 
-  message(4, "CDDB: READ response: %s", resp);
+  log_message(4, "CDDB: READ response: %s", resp);
 
   if (code[0] == 2) {
     if ((localRecordFd = createLocalCddbFile(category, diskId)) == -2) {
-      message(-1, "Existing local CDDB record for %s/%s will not be overwritten.", category, diskId);
+      log_message(-1, "Existing local CDDB record for %s/%s will not be overwritten.", category, diskId);
     }
     if (readDbEntry(localRecordFd) != 0) {
-      message(-2, "CDDB: Received invalid database entry.");
+      log_message(-2, "CDDB: Received invalid database entry.");
       goto fail;
     }
   }
   else {
-    message(-2, "CDDB: READ failed: %s", resp);
+    log_message(-2, "CDDB: READ failed: %s", resp);
     goto fail;
   }
   
@@ -819,7 +799,6 @@ int Cddb::readDb(const char *category, const char *diskId, CddbEntry **entry)
 void Cddb::shutdown()
 {
   const char *resp;
-  const char *args[1];
   int code[3];
 
   if (fd_ < 0) 
@@ -830,18 +809,16 @@ void Cddb::shutdown()
     return;
   }
 
-  args[0] = "quit";
-
-  if (sendCommand(1, args) == 0) {
+  if (sendCommand({"quit"}) == 0) {
     if ((resp = getServerResponse(code)) == NULL) {
-      message(-1, "CDDB: EOF while waiting for QUIT response.");
+      log_message(-1, "CDDB: EOF while waiting for QUIT response.");
     }
     else {
-      message(4, "CDDB: QUIT response: %s", resp);
+      log_message(4, "CDDB: QUIT response: %s", resp);
     }
   }
   else {
-    message(-1, "CDDB: Failed to send QUIT command.");
+    log_message(-1, "CDDB: Failed to send QUIT command.");
   }
 
   closeConnection();
@@ -903,46 +880,41 @@ int Cddb::addAsCdText(Toc *toc)
   toc->cdTextLanguage(0, 9);
 
   if (haveTitle) {
-    item = new CdTextItem(CdTextItem::CDTEXT_TITLE, 0,
-      (cddbEntry_->diskTitle != NULL) ? cdTextFilter(cddbEntry_->diskTitle) : ""); 
-
+    item = createItem(CdTextItem::PackType::TITLE,
+                      (cddbEntry_->diskTitle != NULL) ? cdTextFilter(cddbEntry_->diskTitle) : ""); 
     toc->addCdTextItem(0, item);
   }
 
   if (havePerformer) {
-    item = new CdTextItem(CdTextItem::CDTEXT_PERFORMER, 0,
-      (cddbEntry_->diskArtist != NULL) ? cdTextFilter(cddbEntry_->diskArtist) : "");
-
+    item = createItem(CdTextItem::PackType::PERFORMER,
+                      (cddbEntry_->diskArtist != NULL) ? cdTextFilter(cddbEntry_->diskArtist) : "");
     toc->addCdTextItem(0, item);
   }
 
   if (haveMessage) {
-    item = new CdTextItem(CdTextItem::CDTEXT_MESSAGE, 0,
-      (cddbEntry_->diskExt != NULL) ? cdTextFilter(cddbEntry_->diskExt) : "");
-
+    item = createItem(CdTextItem::PackType::MESSAGE,
+                      (cddbEntry_->diskExt != NULL) ? cdTextFilter(cddbEntry_->diskExt) : "");
     toc->addCdTextItem(0, item);
   }
     
   for (trun = 0; trun < toc->nofTracks(); trun++) {
+
+    item = nullptr;
     if (haveTitle) {
-      
-      item = new CdTextItem(CdTextItem::CDTEXT_TITLE, 0,
-        (trun < cddbEntry_->ntracks && cddbEntry_->trackTitles[trun] != NULL) ? cdTextFilter(cddbEntry_->trackTitles[trun]) : ""); 
-
+      item = createItem(CdTextItem::PackType::TITLE,
+                        (trun < cddbEntry_->ntracks && cddbEntry_->trackTitles[trun] != NULL) ?
+                        cdTextFilter(cddbEntry_->trackTitles[trun]) : "");
       toc->addCdTextItem(trun + 1, item);
     }
-
     if (havePerformer) {
-      item = new CdTextItem(CdTextItem::CDTEXT_PERFORMER, 0,
-        (cddbEntry_->diskArtist != NULL) ? cdTextFilter(cddbEntry_->diskArtist) : "");
-
+      item = createItem(CdTextItem::PackType::PERFORMER,
+                        (cddbEntry_->diskArtist != NULL) ? cdTextFilter(cddbEntry_->diskArtist) : "");
       toc->addCdTextItem(trun + 1, item);
     }
-
     if (haveMessage) {
-      item = new CdTextItem(CdTextItem::CDTEXT_MESSAGE, 0,
-        (trun < cddbEntry_->ntracks && cddbEntry_->trackExt[trun] != NULL) ? cdTextFilter(cddbEntry_->trackExt[trun]) : "");
-
+      item = createItem(CdTextItem::PackType::MESSAGE,
+                        (trun < cddbEntry_->ntracks && cddbEntry_->trackExt[trun] != NULL) ?
+                        cdTextFilter(cddbEntry_->trackExt[trun]) : "");
       toc->addCdTextItem(trun + 1, item);
     }
   }
@@ -973,12 +945,12 @@ const char *Cddb::readLine()
     ret = select(fd_ + 1, &readFds, NULL, NULL, &tv);
 
     if (ret == 0) {
-      message(-2, "CDDB: Timeout while reading data.");
+      log_message(-2, "CDDB: Timeout while reading data.");
       return NULL;
     }
     
     if (ret < 0) {
-      message(-2, "CDDB: Error while waiting for data: %s", strerror(errno));
+      log_message(-2, "CDDB: Error while waiting for data: %s", strerror(errno));
       return NULL;
     }
 
@@ -990,7 +962,7 @@ const char *Cddb::readLine()
     }
     
     if (ret < 0) {
-      message(-2, "CDDB: Error while reading data: %s", strerror(errno));
+      log_message(-2, "CDDB: Error while reading data: %s", strerror(errno));
       return NULL;
     }
     
@@ -1019,7 +991,7 @@ const char *Cddb::readLine()
   for (pos = strlen(s) - 1; pos >= 0 && isspace(s[pos]); pos--) 
     s[pos] = 0;
 
-  message(5, "CDDB: Data read: %s", s);
+  log_message(5, "CDDB: Data read: %s", s);
 
   return s;
 }
@@ -1062,106 +1034,53 @@ const char *Cddb::getServerResponse(int code[3])
  * Return: 0: OK
  *         1: communication error occured.
  */
-int Cddb::sendCommand(int nargs, const char *args[])
+int Cddb::sendCommand(const vector<string>& args)
 {
-  char portBuf[20];
-  int len = 0;
   int err = 0;
-  char *cmd, *p;
-  char *httpCmd = NULL;
-  int run, ret;
-  struct timeval tv;
-  fd_set writeFds;
+  string cmd;
 
-  // build command line
-  for (run = 0; run < nargs; run++)
-    len += strlen(args[run]) + 1;
-
-  cmd = new char[len + 1];
-  *cmd = 0;
-
-  for (run = 0; run < nargs; run++) {
-    strcat(cmd, args[run]);
-    if (run != nargs - 1) {
-      if (httpMode_)
-	strcat(cmd, "+");
-      else
-	strcat(cmd, " ");
-    }
+  for (const auto& s : args) {
+    if (!cmd.empty())
+      cmd += (httpMode_ ? "+" : " ");
+    cmd += s;
   }
 
   if (httpMode_) {
+
+    std::ostringstream ss;
     if (selectedServer_->httpProxyServer != NULL) {
-      sprintf(portBuf, ":%u", selectedServer_->port);
-      httpCmd = strdupvCC("GET http://", selectedServer_->server,
-			portBuf, selectedServer_->httpCgiBin, 
-			"?cmd=", cmd, httpCmd_, " HTTP/1.0\r\n", 
-			"Host: ", selectedServer_->server, "\r\n",
-			httpData_, "\r\n", NULL);
+      ss <<  "GET http://" << selectedServer_->server;
+      ss <<  selectedServer_->port << selectedServer_->httpCgiBin;
+      ss << "?cmd=" <<  cmd << " HTTP/1.0\r\n";
+      ss << "Host: " << selectedServer_->server << "\r\n";
+      ss << httpData_ << "\r\n";
     }
     else {
-      httpCmd = strdupvCC("GET ", selectedServer_->httpCgiBin, 
-			  "?cmd=", cmd, httpCmd_, " HTTP/1.0\r\n", 
-			  "Host: ", selectedServer_->server, "\r\n",
-			  httpData_, "\r\n", NULL);
+      ss << "GET " << selectedServer_->httpCgiBin
+         << "?cmd=" << cmd << " HTTP/1.0\r\n"
+         << "Host: " << selectedServer_->server << "\r\n"
+         << httpData_ << "\r\n";
     }
 
-    delete[] cmd;
-    cmd = httpCmd;
-    httpCmd = NULL;
-
-    message(4, "CDDB: Sending command '%s'...", cmd);
+    cmd = ss.str();
+    log_message(4, "CDDB: Sending command '%s'...", cmd.c_str());
   }
   else {
-    message(4, "CDDB: Sending command '%s'...", cmd);
-    
-    strcat(cmd, "\n");
+    log_message(4, "CDDB: Sending command '%s'...", cmd.c_str());
+    cmd += "\n";
   }
 
-  len = strlen(cmd);
-  p = cmd;
+  auto ret = ::write(fd_, cmd.c_str(), cmd.size());
 
-  while (len > 0) {
-    FD_ZERO(&writeFds);
-    FD_SET(fd_, &writeFds);
-
-    tv.tv_sec = timeout_;
-    tv.tv_usec = 0;
-
-    ret = select(fd_ + 1, NULL, &writeFds, NULL, &tv);
-
-    if (ret == 0) {
-      message(-2, "CDDB: Timeout while sending data.");
-      err = 1; goto fail;
-    }
-    
-    if (ret < 0) {
-      message(-2, "CDDB: Error while waiting for send: %s", strerror(errno));
-      err = 1; goto fail;
-    }
- 
-    ret = write(fd_, p, 1);
-
-    if (ret < 0) {
-      message(-2, "CDDB: Failed to send command '%s': %s", cmd,
-	      strerror(errno));
-      err = 1; goto fail;
-    }
-
-    if (ret != 1) {
-      message(-2, "CDDB: Failed to send command '%s'.", cmd);
-      err = 1; goto fail;
-    }
-
-    len--;
-    p++;
+  if (ret != cmd.size()) {
+    log_message(-2, "CDDB: Failed to send command '%s': %s", cmd.c_str(),
+                strerror(errno));
+    err = 1; goto fail;
   }
 
-  message(4, "CDDB: Ok.");
+  log_message(4, "CDDB: Ok.");
 
   fail:
-  delete[] cmd;
-
   return err;
 }
 
@@ -1247,7 +1166,7 @@ static void convertEscapeSequences(const char *in, char *out)
 static int parseQueryResult(char *line, char *category, char *diskId,
 			    char *title)
 {
-  char *sep = " \t";
+  const char *sep = " \t";
   char *p;
   
   if ((p = strtok(line, sep)) != NULL) {
@@ -1307,7 +1226,7 @@ int Cddb::readDbEntry(int localRecordFd)
 
   
   while ((resp = readLine()) != NULL && strcmp(resp, ".") != 0) {
-    message(4, "CDDB: READ data: %s", resp);
+    log_message(4, "CDDB: READ data: %s", resp);
 
     if (localRecordFd >= 0) {
       // save to local CDDB record file
@@ -1388,7 +1307,7 @@ int Cddb::readDbEntry(int localRecordFd)
   }
 
   if (resp == NULL) {
-    message(-2, "CDDB: EOF while reading database entry.");
+    log_message(-2, "CDDB: EOF while reading database entry.");
     goto fail;
   }
 
@@ -1422,64 +1341,64 @@ int Cddb::readDbEntry(int localRecordFd)
 
 int Cddb::createLocalCddbFile(const char *category, const char *diskId)
 {
-  char *categoryDir = NULL;
-  char *recordFile = NULL;
+  std::string categoryDir;
+  std::string recordFile;
   struct stat sbuf;
   int ret;
   int fd = -1;
 
-  if (localCddbDirectory_ == NULL)
+  if (localCddbDirectory_.empty())
     return -1;
 
-  ret = stat(localCddbDirectory_, &sbuf);
+  ret = stat(localCddbDirectory_.c_str(), &sbuf);
 
   if (ret != 0 && errno == ENOENT) {
-    message(-1, "CDDB: Local CDDB directory \"%s\" does not exist.",
-	    localCddbDirectory_);
+    log_message(-1, "CDDB: Local CDDB directory \"%s\" does not exist.",
+                localCddbDirectory_.c_str());
     return -1;
   }
   else if (ret == 0) {
     if (!S_ISDIR(sbuf.st_mode)) {
-      message(-2, "CDDB: \"%s\" is not a directory.", localCddbDirectory_);
+      log_message(-2, "CDDB: \"%s\" is not a directory.", localCddbDirectory_.c_str());
       return -1;
     }
   }
   else {
-    message(-2, "CDDB: stat of \"%s\" failed: %s", localCddbDirectory_,
+    log_message(-2, "CDDB: stat of \"%s\" failed: %s", localCddbDirectory_.c_str(),
 	    strerror(errno));
     return -1;
   }
 
-  categoryDir = strdup3CC(localCddbDirectory_, "/", category);
+  categoryDir = localCddbDirectory_ + "/" + category;
 
-  ret = stat(categoryDir, &sbuf);
+  ret = stat(categoryDir.c_str(), &sbuf);
 
   if (ret != 0 && errno == ENOENT) {
-    if (mkdir(categoryDir, 0777) != 0) {
-      message(-2, "CDDB: Cannot create directory \"%s\": %s", categoryDir,
+    if (mkdir(categoryDir.c_str(), 0777) != 0) {
+      log_message(-2, "CDDB: Cannot create directory \"%s\": %s", categoryDir.c_str(),
 	      strerror(errno));
       goto fail;
     }
   }
   else if (ret == 0) {
     if (!S_ISDIR(sbuf.st_mode)) {
-      message(-2, "CDDB: \"%s\" is not a directory.", categoryDir);
+      log_message(-2, "CDDB: \"%s\" is not a directory.", categoryDir.c_str());
     }
   }
   else {
-    message(-2, "CDDB: stat of \"%s\" failed: %s", categoryDir,
+    log_message(-2, "CDDB: stat of \"%s\" failed: %s", categoryDir.c_str(),
 	    strerror(errno));
     goto fail;
   }
 
-  recordFile = strdup3CC(categoryDir, "/", diskId);
+  recordFile = categoryDir + "/" + diskId;
   
-  ret = stat(recordFile, &sbuf);
+  ret = stat(recordFile.c_str(), &sbuf);
 
   if (ret != 0 && errno == ENOENT) {
-    if ((fd = open(recordFile, O_WRONLY|O_CREAT, 0666)) < 0) {
-      message(-2, "CDDB: Cannot create CDDB record file \"%s\": %s",
-	      recordFile, strerror(errno));
+    if ((fd = open(recordFile.c_str(), O_WRONLY|O_CREAT, 0666)) < 0) {
+      log_message(-2, "CDDB: Cannot create CDDB record file \"%s\": %s",
+                  recordFile.c_str(), strerror(errno));
       fd = -1;
       goto fail;
     }
@@ -1489,15 +1408,19 @@ int Cddb::createLocalCddbFile(const char *category, const char *diskId)
     goto fail;
   }
   else {
-    message(-2, "CDDB: stat of \"%s\" failed: %s", categoryDir,
+    log_message(-2, "CDDB: stat of \"%s\" failed: %s", categoryDir.c_str(),
 	    strerror(errno));
     goto fail;
   }
   
  fail:
 
-  delete[] categoryDir;
-  delete[] recordFile;
-
   return fd;
 }
+
+CdTextItem* Cddb::createItem(CdTextItem::PackType ptype, const char* text)
+{
+    auto item = new CdTextItem(ptype, 0);
+    item->setText(text);
+    return item;
+}
